#!/bin/bash
# adriane-common.sh
# Global and local configuration, macros for ADRIANE
# (C) Klaus Knopper 2014

# Global variables
PATH="/bin:/sbin:/usr/bin:/usr/sbin:/usr/X11R6/bin:/usr/local/bin:/usr/local/sbin:$HOME/bin"
export PATH

export ADRIANE="1.7"

NOOK=""
DEFAULTWIDTH=""
DEFAULTHEIGHT=""
TERMOPTS=""
DIALOG="dialog"
# ADRIANE under X
if [ -n "$DISPLAY" ] && type -t Xdialog >/dev/null 2>&1; then
 DIALOG="Xdialog 1>&2 2>/dev/null --stdout"
 export XDIALOG_HIGH_DIALOG_COMPAT=1
 export XDIALOG_NO_GMSGS=1
 DEFAULTWIDTH=75
 DEFAULTHEIGHT=18
else
 # Quick check for support of "--no-ok" in dialog
 case "$($DIALOG --help 2>&1)" in *--no-ok*) NOOK="--no-ok";; esac
 export DIALOGOPTS="--begin 0 0 --no-shadow"
 [ -f /etc/adriane/dialogrc ] && export DIALOGRC="/etc/adriane/dialogrc"
fi

# Defaults, overwritten by /etc/adriane/adriane.conf and local config
COLUMNS="80"
LINES="25"
# Default text editor
EDITOR="nano -t"
VIEWER="nano -t -v"
ESPEAK_PITCH="80"
ESPEAK_SPEED="100"
COMMAND_WWW="elinks"
COMMAND_EMAIL="mutt"
COMMAND_CHAT="ysm"
COMMAND_SPEAK="spd-say"

# Global configuration, language
if [ -r "/etc/sysconfig/i18n" ]; then
 . "/etc/sysconfig/i18n"
elif [ -r "/etc/default/locale" ]; then
 . "/etc/default/locale"
fi

# Source common functions & global variables
# (/etc/adriane/adriane.conf, $HOME/.adriane/adriane.conf)

# Global ADRIANE configuration
[ -r "/etc/adriane/adriane.conf" ] && . "/etc/adriane/adriane.conf"

# User configuration
[ -r "$HOME/.adriane/adriane.conf" ] && . "$HOME/.adriane/adriane.conf"

# TODO: Bugfix: Xdialog problems with UTF-8?

TMP="/tmp/adriane.tmp.$$"

# Localization is a little more complicated here,
# since adriane-common is included by many files.
TEXTDOMAIN_SAVE="$TEXTDOMAIN"
export TEXTDOMAIN="adriane-common"
MSG_HELP=$(gettext "Help")
MSG_ERROR_HELP=$(gettext "Help text not available yet.")
MSG_NO=$(gettext "No")
MSG_YES=$(gettext "Yes")
MSG_FULLMESSAGE=$(gettext "Show full message")
MSG_ERROR=$(gettext "Error")
MSG_SELECT=$(gettext "Select")
MSG_QUIT=$(gettext "Quit")
MSG_BACK=$(gettext "Back")
MSG_EDIT=$(gettext "Edit")
MSG_MANAGE_FILES=$(gettext "Manage saved files")
MSG_CANCEL=$(gettext "Cancel")
MSG_DIRECTORY=$(gettext "Directory")
MSG_MOUNTERROR=$(gettext "error mounting %s")
MSG_MOUNTED=$(gettext "mounted")
MSG_NOTMOUNTED=$(gettext "not mounted")
MSG_MOUNTIT=$(gettext "mount contents of %s")
MSG_UMOUNTIT=$(gettext "release mounted directory %s")
MSG_BROWSEHOME=$(gettext "browse Home Directory")
MSG_BROWSEFILES=$(gettext "browse files in %s")
MSG_PLAYFILE=$(gettext "play file %s")
MSG_EDITFILE=$(gettext "edit file %s")
MSG_DELETEFILE=$(gettext "delete file %s")
MSG_SEARCHING=$(gettext "searching...")
MSG_COPY_FILE=$(gettext "copy file to this directory")
MSG_UPPER_DIR=$(gettext "upper directory")
MSG_HIDEDOTFILES=$(gettext 'hide ".*" files')
MSG_SHOWDOTFILES=$(gettext 'show ".*" files')
MSG_RMDIR=$(gettext "delete this directory and its contents")
MSG_MKDIR=$(gettext "add a new directory here")
MSG_ERROR_DIRECTORY_NOT_WRITABLE=$(gettext "Directory %s is not writable")
MSG_ENTER_NEW_DIRECTORY_NAME=$(gettext "Enter name for new directory")
MSG_ERROR_CANNOT_CREATE_DIRECTORY=$(gettext "error creating directory %s")
MSG_OK_DIRECTORY_CREATED=$(gettext "directory %s created successfully")
MSG_REALLY_DELETE_DIRECTORY=$(gettext "Really remove directory %s and all of its contents? [Y/n] ")
MSG_ERROR_CANNOT_REMOVE_DIRECTORY=$(gettext "error removing directory %s")
MSG_OK_DIRECTORY_REMOVED=$(gettext "directory %s removed successfully")
export TEXTDOMAIN="$TEXTDOMAIN_SAVE"

# Functions

# Delete these files with bailout when finished. Can be extended later.
TMPFILES=( "$TMP" "$TMP.err" "$TMP.pass" "$TMP.list" "$TMP.msg" "$TMP.pnm" "$TMP.txt" "$TMP.progress" "$TMP.name" "$TMP.tmp" "$TMP.common" "$TMP.yesno" )

# Add file to delete list if not already present
addtmpfile(){
 local file
 for file in "$@"; do
  local i
  for ((i=0;i<${#TMPFILES[@]}]; i++)); do
   case "$file" in "${TMPFILES[$i]}]") continue 2;; esac
  done
  TMPFILES[${#TMPFILES[@]}]="$file"
 done
}

bailout(){
 rm -f "${TMPFILES[@]}"
 exit $1
}

trap bailout 1 2 3 15

if [ -z "$DISPLAY" ]; then
 TTY="$(tty 2>/dev/null)"
 STTYOPTS=""
 case "$TTY" in /dev/tty[0-9]*) CONSOLE="$TTY";; *) CONSOLE="";; esac
 if [ -n "$CONSOLE" ]; then
 # Make sure we can write on the console
  [ -w "$CONSOLE" ] || sudo chmod 666 "$CONSOLE"
  # Set -ixon to avoid accidential console freezes by Control-S
  [ -n "$CONSOLE" ] && STTYOPTS="-ixon"
 fi
 # Set braille device friendly screen size
 if [ "$LINES" -gt "0" ] >/dev/null 2>&1; then
  export LINES
  [ -n "$CONSOLE" ] && STTYOPTS="$STTYOPTS rows $LINES"
 fi
 if [ "$COLUMNS" -gt "0" ] >/dev/null 2>&1; then
  export COLUMNS
   [ -n "$CONSOLE" ] && STTYOPTS="$STTYOPTS cols $COLUMNS"
 fi
 [ -n "$TERM" ] && export TERM
 [ -n "$STTYOPTS" ] && stty $STTYOPTS
 [ -n "$CONSOLE" -a -n "$TERMOPTS" ] && type -p setterm >/dev/null 2>&1 && setterm $TERMOPTS
else
 EDITOR="rxvt -fn 10x20 -e $EDITOR"
fi

[ -n "$EDITOR" ]      && export EDITOR
[ -n "$VIEWER" ]      && export VIEWER
[ -n "$LANG" ]        && export LANG
[ -n "$LANGUAGE" ]    && export LANGUAGE
[ -n "$LINGUAS" ]     && export LINGUAS
[ -n "$LC_LANG" ]     && export LC_LANG
[ -n "$LC_MESSAGES" ] && export LC_MESSAGES

# Common functions
say(){
 if [ -z "$DISPLAY" ]; then
  spd-say -P message "$1" >/dev/null 2>&1
 fi
}

# Produce a continuous number from anything that looks like a number.
numbertostderr(){
 echo "$1" 2>/dev/null | tr -d '[:alpha:][:blank:][:cntrl:][:punct:]' >&2
 return 0
}

# displayfile file title
# (Mis)use menu widget to display a file. --editbox does not do the same.
displayfile(){
 local TEXTDOMAINDIR="/usr/share/locale"
 local TEXTDOMAIN="adriane-common"
 local text=()
 local count=0
 local lang="${LANG%%_*}"
 if [ -r "$1" ]; then
  while true; do
   text[$((count++))]=""
   read -r text[$((count++))] || break
  done < "$1"
 $DIALOG --title "$2" --no-cancel --visit-items --menu " " 23 "${DEFAULTWIDTH:-$COLUMNS}" 16 "${text[@]}" 2>/dev/null
 fi
 return 0
}

# Context-sensitive help (i.e. depends on how we were called)
# Unfinished; At least show the most important commands.
adriane_help(){
 local TEXTDOMAINDIR="/usr/share/locale"
 local TEXTDOMAIN="adriane-common"
 local text=()
 local count=0
 local lang="${LANG%%_*}"
 case "$lang" in C) lang="en";; esac
 local module=""
 if [ -n "$ADRIANE_MODULE" ]; then
  module="-$ADRIANE_MODULE"
 fi

 local helpfile="/usr/share/doc/adriane-doc/help"$module"-"$lang".txt"
 if [ -r "$helpfile" ]; then
  displayfile "$helpfile" "$MSG_HELP"
 else
  error "$MSG_ERROR_HELP"
 fi
 return 0
}

# progress "Message" seconds_expected
progress(){
local TEXTDOMAINDIR="/usr/share/locale"
local TEXTDOMAIN="adriane-common"
if [ -n "$DISPLAY" ]; then
 if [ -n "$1" ]; then
  # Show progressbar
  touch "$TMP.progress"
  expected="$2"
  [ -n "$expected" ] && ratio="$((expected/10))+2" || ratio=4
  status=0
  ( while [ -e "$TMP.progress" ]; do echo "$(((status=(10000-status)/ratio+status)/100))"; sleep 1; done | $DIALOG --title "$1" --gauge "" 5 "${DEFAULTWIDTH:-75}" 0 ) &
 else
  # Kill previous
  rm -f "$TMP".progress ; wait
 fi
else
 [ -n "$1" ] && { sleep 1; echo -n -e "\r\033[K$1 "; } || echo -n -e "\r\033[K"
fi
 return 0
}

# blocking message
notify(){
 local TEXTDOMAINDIR="/usr/share/locale"
 local TEXTDOMAIN="adriane-common"
 local NOTIFY_TIMEOUT="$2"
 [ -n "$NOTIFY_TIMEOUT" ] || NOTIFY_TIMEOUT=4
 usleep 250000 >/dev/null 2>&1
 $DIALOG --title "$TITLE" $NOOK --no-cancel --timeout "$NOTIFY_TIMEOUT" --visit-items --menu " " 0 "${DEFAULTWIDTH:-$COLUMNS}" 2 "" " $1 " 2>/dev/null
 usleep 250000 >/dev/null 2>&1
 return 0
}

# yesno "text" [defaultno] returns 0 on yes, 1 on no.
yesno(){
 local TEXTDOMAINDIR="/usr/share/locale"
 local TEXTDOMAIN="adriane-common"
 local RC=0
 local a=""
 if [ -n "$DISPLAY" ]; then
  $DIALOG --title "$TITLE" $NOOK --no-cancel --visit-items --menu " " 0 "$COLUMNS" 3 y "$1" n "$MSG_NO" 2>"$TMP".yesno || { rm -f "$TMP".yesno; return 1; }
  read a <"$TMP".yesno
  case "$a" in n) RC="1";; esac
  rm -f "$TMP".yesno
 else
  while true; do
   read -n 1 -s -r -p "$1" a
   if [ -z "$a" ]; then # empty answer, use default
    [ -n "$2" ] && RC="1"
    break
   fi
   case "$a" in [nN]|) RC="1"; break;;
               [jJyYsShH]) RC="0"; break;;
   esac
  done
  usleep 250000 >/dev/null 2>&1
  [ "$RC" = "0" ] && echo -n -e "\n\r $MSG_YES" || echo -n -e "\n\r $MSG_NO"
  sleep 1
  echo ""
 fi
 return "$RC"
}

# prompt message that waits
prompt(){
 local TEXTDOMAINDIR="/usr/share/locale"
 local TEXTDOMAIN="adriane-common"
 usleep 250000 >/dev/null 2>&1
 $DIALOG --title "$TITLE" $NOOK --no-cancel --visit-items --menu " " 0 "${DEFAULTWIDTH:-$COLUMNS}" 2 "" " $1 " 2>/dev/null || return 1
 return 0
}

# error shortmessage [file contaning full message]
error(){
 local TEXTDOMAINDIR="/usr/share/locale"
 local TEXTDOMAIN="adriane-common"
 local VIEWBUTTON=()
 usleep 250000 >/dev/null 2>&1
 $DIALOG --title "$MSG_ERROR" $NOOK --no-cancel --visit-items --menu " " 0 "${DEFAULTWIDTH:-$COLUMNS}" 2 "" " $1 " 2>/dev/null || return 0
 [ -s "$2" ] && displayfile "$2" "$MSG_ERROR" 2>/dev/null
 return 0
}

# Main Menu without ANY buttons
# mainmenu
mainmenu(){
 local TEXTDOMAINDIR="/usr/share/locale"
 local TEXTDOMAIN="adriane-common"
 local SELECTEDITEM=()
 [ -n "$DEFAULTITEM" ] && SELECTEDITEM=( "--default-item" "$DEFAULTITEM" ) 
 $DIALOG --title "$TITLE" $NOOK --no-cancel --ok-label "$MSG_SELECT" --cancel-label "$MSG_QUIT" "${SELECTEDITEM[@]}" --visit-items --menu " " 0 "${DEFAULTWIDTH:-$COLUMNS}" 0 "$@" || return 1
 return 0
}

# submenu item1 desc1 item2 desc2 ...
submenu(){
 local TEXTDOMAINDIR="/usr/share/locale"
 local TEXTDOMAIN="adriane-common"
 local SELECTEDITEM=()
 [ -n "$DEFAULTITEM" ] && SELECTEDITEM=( "--default-item" "$DEFAULTITEM" ) 
 usleep 250000 >/dev/null 2>&1
 $DIALOG --title "$TITLE" $NOOK --no-cancel --ok-label "$MSG_SELECT" --cancel-label "$MSG_BACK" "${SELECTEDITEM[@]}" --visit-items --menu " " 0 "${DEFAULTWIDTH:-$COLUMNS}" 0 "$@" || return 1
 return 0
}

# editmenu item1 desc1 item2 desc2 ...
editmenu(){
 local TEXTDOMAINDIR="/usr/share/locale"
 local TEXTDOMAIN="adriane-common"
 local SELECTEDITEM=()
 [ -n "$DEFAULTITEM" ] && SELECTEDITEM=( "--default-item" "$DEFAULTITEM" ) 
 usleep 250000 >/dev/null 2>&1
 $DIALOG --title "$TITLE" --ok-label "$MSG_EDIT" --cancel-label "$MSG_BACK" "${SELECTEDITEM[@]}" --visit-items --menu "" 0 "${DEFAULTWIDTH:-$COLUMNS}" 0 "$@" || return 1
 return 0
}

# textfield [default]
textfield(){
 local TEXTDOMAINDIR="/usr/share/locale"
 local TEXTDOMAIN="adriane-common"
 usleep 250000 >/dev/null 2>&1
 $DIALOG --title "$TITLE" $NOOK --no-cancel --cancel-label "$MSG_BACK" --inputbox " " 9 "${DEFAULTWIDTH:-$COLUMNS}" "$1" || return 1
 return 0
}

# textinput var prompt-text
textinput(){
 local TEXTDOMAINDIR="/usr/share/locale"
 local TEXTDOMAIN="adriane-common"
 if [ -n "$DISPLAY" ]; then
  $DIALOG --title "$TITLE" $NOOK --no-cancel --cancel-label "$MSG_BACK" --inputbox "$2" 9 "${DEFAULTWIDTH:-$COLUMNS}" "$1" 2>"$TMP".textinput || { rm -f "$TMP".textinput; return 1; }
  read "$1" < "$TMP".textinput; rm -f "$TMP".textinput
 else
  usleep 250000 >/dev/null 2>&1
  read -e -p "$2" "$1"
 fi
 return "$?"
}

# Add subdirectory to selected directory
# make_directory new_dir_variable directory
make_directory(){
 local make_directory_name=""
 if [ ! -w "$2" ]; then
  error "$(printf "$MSG_ERROR_DIRECTORY_NOT_WRITABLE" "$2")"
  return 1
 fi
 textinput make_directory_name "${MSG_ENTER_NEW_DIRECTORY_NAME}: " || return 2
 local make_directory_base="${2##*/}"
 rm -f "$TMP".err
 if ! mkdir "$2/$make_directory_name" 2>"$TMP".err; then
  error "$(printf "$MSG_ERROR_CANNOT_CREATE_DIRECTORY" "$make_directory_base/$make_directory_name")" "$TMP".err
  rm -f "$TMP".err
  return 3
 fi
 rm -f "$TMP".err
 notify "$(printf "$MSG_OK_DIRECTORY_CREATED" "$make_directory_name")"
 read -r "$1" <<.
$2/$make_directory_name
.
 return 0
}

delete_directory(){
 if [ ! -w "$1" ]; then
  error "$(printf "$MSG_ERROR_DIRECTORY_NOT_WRITABLE" "$1")"
  return 1
 fi
 local delete_directory_base="${1##*/}"
 yesno "$(printf "$MSG_REALLY_DELETE_DIRECTORY" "$delete_directory_base")" || return 2
 rm -f "$TMP".err
 if ! rm -rf "$1" 2>"$TMP".err; then
  error "$(printf "$MSG_ERROR_CANNOT_REMOVE_DIRECTORY" "$delete_directory_base")" "$TMP".err
  rm -f "$TMP".err
  return 3
 fi
 rm -f "$TMP".err
 notify "$(printf "$MSG_OK_DIRECTORY_REMOVED" "$delete_directory_base")"
 return 0
}

# selectfile variable [directory]
# SELECTFILE_ALLOW_MANAGE controls appearance of "manage files" item
# SELECTFILE_ALLOW_NEWFILE controls appearance of "copy file to this directory" item
# SELECTFILE_ALLOW_RMDIR controls appearance of "delete this directory and its contents" item
# SELECTFILE_ALLOW_MKDIR controls appearance of "add a new directory here" item
# SELECTFILE_CHECKFILE selects a directory that contains file $SELECTFILE_CHECKFILE
selectfile(){
 local TEXTDOMAINDIR="/usr/share/locale"
 local TEXTDOMAIN="adriane-common"
 local SELECT_SHOWDOTFILES=""
 local SELECTEDITEM=()
 local selectfile_dir="$2"
 [ -d "$selectfile_dir" ] || selectfile_dir="."
 while true; do
  [ -n "$selectfile_dir" ] || selectfile_dir="/"
  if [ -n "$SELECTFILE_CHECKFILE" ] && ls -1 "$selectfile_dir" 2>/dev/null | grep -q "^$SELECTFILE_CHECKFILE\$" 2>/dev/null; then
   read "$1" <<EOT
$selectfile_dir
EOT
   return 0
  fi
  unset FILE
  local FILE=("${selectfile_dir%/*}")
  local fcount="${#FILE[@]}"
  unset LIST
  local LIST=()
  local lcount="${#LIST[@]}"
  LIST[$((lcount++))]="."
  if [ -n "$SELECT_SHOWDOTFILES" ]; then
   LIST[$((lcount++))]="$MSG_HIDEDOTFILES"
  else 
   LIST[$((lcount++))]="$MSG_SHOWDOTFILES"
  fi
  if [ -n "$SELECTFILE_ALLOW_RMDIR" -a "$selectfile_dir" != "/" ]; then
   LIST[$((lcount++))]="d"
   LIST[$((lcount++))]="$MSG_RMDIR"
  fi
  if [ -n "$SELECTFILE_ALLOW_MKDIR" ]; then
   LIST[$((lcount++))]="a"
   LIST[$((lcount++))]="$MSG_MKDIR"
  fi
  LIST[$((lcount++))]="0"
  LIST[$((lcount++))]="$(printf "%-63.63s [DIR]" "$MSG_UPPER_DIR")"
  local SELECT_DEFAULTITEM="$DEFAULTITEM"
  if [ -n "$SELECTFILE_ALLOW_NEWFILE" ]; then
   LIST[$((lcount++))]="c"
   LIST[$((lcount++))]="$MSG_COPY_FILE"
   [ -n "$SELECT_DEFAULTITEM" ] || SELECT_DEFAULTITEM="c"
  fi
  if [ -n "$SELECTFILE_ALLOW_MANAGE" ]; then
   LIST[$((lcount++))]="m"
   LIST[$((lcount++))]="$MSG_MANAGE_FILES"
   [ -n "$SELECT_DEFAULTITEM" ] || SELECT_DEFAULTITEM="m"
  fi
  local selectfile_file=""
  local SELECTION
  local shortname
  progress "$MSG_SEARCHING" 4
  while read selectfile_file; do
   [ -n "$selectfile_file" ] || continue
   [ "$selectfile_file" = "$selectfile_dir" ] && continue
   shortname="${selectfile_file##*/}"
   case "$shortname" in
    ..) continue;;
    .*) [ -n "$SELECT_SHOWDOTFILES" ] || continue;;
   esac
   FILE[$fcount]="$selectfile_file"
   # Strip extension?
   # shortname="${shortname%.*}"
   if [ -d "$selectfile_file" ]; then
    shortname="$(printf "%-63.63s [DIR]" "$shortname")"
   else
    shortname="$(printf "%-.69s" "$shortname")"
   fi
   if [ -z "$SELECTFILE_CHECKFILE" -o -d "$selectfile_file" ]; then
    LIST[$((lcount++))]="$((fcount++))"
    LIST[$((lcount++))]="$shortname"
   fi
  done <<.
$(find "$selectfile_dir" -maxdepth 1 -type d 2>/dev/null | LC_COLLATE=C sort)
$(find "$selectfile_dir" -maxdepth 1 -type f 2>/dev/null | LC_COLLATE=C sort)
.
  progress ""
  if [ -z "$SELECT_DEFAULTITEM" ]; then
   [ "${#FILE[@]}" -gt 1 ] && SELECT_DEFAULTITEM="1" || SELECT_DEFAULTITEM="0"
  fi
  SELECTEDITEM=( "--default-item" "$SELECT_DEFAULTITEM" )
  rm -f "$TMP"
#  $DIALOG --title "$TITLE" --ok-label "$MSG_SELECT" --cancel-label "$MSG_CANCEL" "${SELECTEDITEM[@]}" --visit-items --menu "$MSG_DIRECTORY $selectfile_dir" 0 "${DEFAULTWIDTH:-$COLUMNS}" 0 "${LIST[@]}" 2>"$TMP".tmp || return 1
  $DIALOG --title "$TITLE" $NOOK --no-cancel "${SELECTEDITEM[@]}" --visit-items --menu "$MSG_DIRECTORY $selectfile_dir" 0 "${DEFAULTWIDTH:-$COLUMNS}" 0 "${LIST[@]}" 2>"$TMP".tmp || return 1
  read -r SELECTION <"$TMP".tmp ; rm -f "$TMP".tmp
  case "$SELECTION" in
   0) # Upper directory
     selectfile_dir="${selectfile_dir%/*}" ;;
   [1-9]*)
     selectfile_file="${FILE[$SELECTION]}"
     if [ -f "$selectfile_file" ]; then
      read -r "$1" <<.
$selectfile_file
.
      return 0
     fi
     if [ -d "$selectfile_file" ]; then
      case "$selectfile_file" in */..) selectfile_dir="${selectfile_dir%/*}";; *) selectfile_dir="$selectfile_file";; esac
     fi
     ;;
    m) ( cd "$selectfile_dir"; mc ) ;;
    c) read -r "$1" <<.
$selectfile_dir
.
       return 0
     ;;
    a) local selectfile_new_dir=""
       if make_directory selectfile_new_dir "$selectfile_dir"; then
        [ -n "$selectfile_new_dir" ] && selectfile_dir="$selectfile_new_dir"
       fi ;;
    d) delete_directory "$selectfile_dir" && selectfile_dir="${selectfile_dir%/*}" ;;
    .) [ -n "$SELECT_SHOWDOTFILES" ] && SELECT_SHOWDOTFILES="" || SELECT_SHOWDOTFILES="true";;
  esac
 done
 return 0
}

# converttotext inputfile outputfile
converttotext(){
 local ft=$(dd if="$i" bs=10k count=1 | awk '/^[Cc]ontent-[Tt]ype: /{print $2}')
 [ -n "$ft" ] || ft=$(dd if="$i" bs=10k count=1 | awk '/^ *<[Hh][Tt][Mm][Ll] */{print $0}')
 [ -n "$ft" ] || ft=$(file "$1")
 case "$ft" in
  *[Hh][Tt][Mm][Ll]*)
   elinks -force-html -dump-width "$((COLUMNS - 1))" -no-connect -no-numbering -no-references $ELINKS_OPTS -dump "$1" > "$2";;
  *[Pp][Dd][Ff]*) pdftotext -layout "$1" "$2" ;;
  *Microsoft\ Office*) catdoc "$1" > "$2" ;;
  *text*) fold -s -w "$((COLUMNS - 1))" "$1" > "$2" ;;
  *) cp "$1" "$2" ;;
 esac
 return "$?"
}

# mountit /dev/device [other mount options]
mountit(){
 rm -f "$TMP.err"
 { type -p pmount >/dev/null 2>&1 && pmount "$1" || mount "$@"; } 2> "$TMP.err"
 local RC="$?"
 [ "$RC" != 0 ] && error "$(printf "$MSG_MOUNTERROR" "$1")" "$TMP.err"
 rm -f "$TMP.err"
 return $RC
}

umountit(){
 type -p pumount >/dev/null 2>&1 && pumount "$1" || umount "$@"
 return $?
}

mounted(){
 local mp=$(awk "/^\/dev\/$1/"'{print $3}' <<EOT
$(mount)
EOT
)
 echo "$mp"
 [ -n "$mp" ] && return 0 || return 1
}

# Size is given in 512 byte sectors in /sys/block/<disk>/<partition>/size
# We follow the manufacturers "renaming" of capacities by calculating
# with divisors of 100 instead of the correct 1024 value, and add 1
# for roundup
calc_size(){
 local calc_size_size="$(($1 / 2))"
 if [ "$calc_size_size" -gt 1000000 ] 2>/dev/null; then
  echo " [$(($calc_size_size / 1000 / 1000 + 1))GB]"
 elif [ "$calc_size_size" -gt 1000 ] 2>/dev/null; then
  echo " [$(($calc_size_size / 1000 + 1))MB]"
 elif [ "$calc_size_size" -gt 0 ] 2>/dev/null; then
  echo " [${calc_size_size}KB]"
 fi
 return "$?"
}

# mount_selection [Variablename_dir] [Variablename_action]
# Returns folder of mounted device (or homedir)
mount_selection(){
 local TEXTDOMAINDIR="/usr/share/locale"
 local TEXTDOMAIN="adriane-common"
 local PARTITION=()
 local DESCRIPTION=()
 local MOUNTED=()
 local ACTIONS=()
 local MOUNTDIR
 local LIST=(h "$MSG_BROWSEHOME")
 local i count p_count selection index
 local disk partition device dev mounted model vendor
 for disk in /sys/block/[hs]d[a-z] /sys/block/sr[0-9]* /sys/block/mmcblk*; do
  [ -d "$disk" ] || continue
  device="${disk##*/}"
  vendor=""
  [ -r "$disk"/device/vendor ] && read vendor <"$disk"/device/vendor
  model=""
  [ -r "$disk"/device/model ] && read model  <"$disk"/device/model
  p_count=0
  for partition in "$disk"/[hs]d[a-z]* "$disk"/mmcblk*; do
   [ -d "$partition" ] || continue
   let p_count++
   read size <"$partition"/size 2>/dev/null
   size="$(calc_size $size 2>/dev/null)"
   dev="${partition##*/}"
   PARTITION[$count]="$dev"
   mounted "$dev" >/dev/null && mounted="${MSG_MOUNTED}: " || mounted=""
   MOUNTED[$count]="$mounted"
   DESCRIPTION[$count]="$(printf "%-5s %s%.16s %.16s (%d)%s" "$dev" "$mounted" "$vendor" "$model" "$p_count" "$size")"
   let count++
  done
  if [ "$p_count" -eq 0 ]; then
   read size <"$disk"/size 2>/dev/null
   size="$(calc_size $size 2>/dev/null)"
   PARTITION[$count]="$device"
   mounted "$device" >/dev/null && mounted="${MSG_MOUNTED}: " || mounted=""
   MOUNTED[$count]="$mounted"
   DESCRIPTION[$count]="$(printf "%-5s %s%.16s %.16s%s" "$device" "$mounted" "$vendor" "$model" "$size")"
   let count++
  fi
 done 
 count=2
 for ((i=0; i < ${#PARTITION[@]}; i++)); do
  LIST[$((count++))]="${PARTITION[$i]}"
  LIST[$((count++))]="${DESCRIPTION[$i]}"
 done
 rm -f "$TMP"
 submenu "${LIST[@]}" 2>"$TMP" || { rm -f "$TMP"; return 1; }
 read selection <"$TMP"; rm -f "$TMP"
 case "$selection" in
  h) [ -n "$1" ] && read -r "$1" <<.
$HOME
.
     [ -n "$2" ] && read -r "$2" <<.
browse
.
     ;;
  *) count=0; index=0
     for ((i=0; i<${#PARTITION[@]}; i++)); do
      if [ "$selection" = "${PARTITION[$i]}" ]; then
       index="$i"; break
      fi
     done
     selection=""
     while true; do
      MOUNTDIR="$(mounted "${PARTITION[$index]}")"
      UMOUNTACTION=( u "$(printf "$MSG_UMOUNTIT" "$MOUNTDIR")" )
      BROWSEACTION=( f "$(printf "$MSG_BROWSEFILES" "$MOUNTDIR")" )
      MOUNTACTION=( m "$(printf "$MSG_MOUNTIT" "/dev/${PARTITION[$index]}")" )
      if [ "$selection" = "f" ]; then
       ACTIONS=( "${UMOUNTACTION[@]}" "${BROWSEACTION[@]}" )
      else
       if [ -n "$MOUNTDIR" ]; then
        ACTIONS=( "${BROWSEACTION[@]}" "${UMOUNTACTION[@]}" )
       else
        ACTIONS=( "${MOUNTACTION[@]}" )
       fi
      fi
      rm -f "$TMP"
      submenu "${ACTIONS[@]}" 2>"$TMP" || { rm -f "$TMP"; break; }
      read selection <"$TMP"; rm -f "$TMP"
      case "$selection" in
       m) mountit "/dev/${PARTITION[$index]}" ;;
       u) umountit "/dev/${PARTITION[$index]}"; break ;;
       f) [ -n "$1" ] && read -r "$1" <<.
$MOUNTDIR
.
          [ -n "$2" ] && read -r "$2" <<.
browse
.
	  return 0;;
      esac
     done
     ;;
 esac
 return 0
}

notyet(){
 error "$@"
 return 0
}

# Set speech dispatcher address, if not already set
test -z "$SPEECHD_ADDRESS" && grep -q -E -i '^[[:space:]]*SocketPath[[:space:]]+"/var/run/speech-dispatcher/speechd.sock"' /etc/speech-dispatcher/speechd.conf >/dev/null 2>&1 && export SPEECHD_ADDRESS="unix_socket:/var/run/speech-dispatcher/speechd.sock"
